//#define MT_IMPLEMENTATION

#include <MT/ors.h>
#include <MT/ors_control.h>
#include <MT/ors_opengl.h>
#include <MT/ors_swift.h>
#include <MT/soc.h>
#include <MT/opengl.h>

const char* USAGE="usage: ./x.exe -orsfile test.ors -dynamic 1 -Hcost 1e-3";

int main(int argn,char **argv){
  MT::initCmdLine(argn,argv);
  cout <<USAGE <<endl;

  //-- setup the simulator, swift, opengl, and the SocAbstraction
  ors::Graph ors;
  ors.init(MT::getParameter<MT::String>("orsfile",MT::String("test.ors")));
  //cout <<"read configuration:\n" <<ors <<endl;

  SwiftModule swift;
  swift.init(ors,.5);
  
  OpenGL gl;
  gl.add(glStandardScene);
  gl.add(ors::glDrawGraph,&ors);
  gl.camera.setPosition(5,-10,10);
  gl.camera.focus(0,0,1);
  gl.watch("loaded configuration - press ENTER");

  uint T=200;
  soc::OrsSocImplementation soc;
  if(MT::getParameter<bool>("dynamic",false)){
    soc.initPseudoDynamic(&ors,&swift,&gl,3.,T);
  }else{
    soc.initKinematic(&ors,&swift,&gl,T);
  }
  soc.os=&std::cout;

  //-- setup the control variables (problem definition)
  ControlVariable *pos = new ControlVariable("position",ors, posCVT,"endeff","<t(0 0 .2)>",0,0,ARR());
  pos->setGainsAsAttractor(20,.2);
  pos->x_target = arr(ors.getName("target")->X.p.v,3);
  
  ControlVariable *col = new ControlVariable("collision",ors, collCVT,0,0,0,0,ARR(.05));
  col->setGains(.5,.0);
  col->prec=1e-0;
  col->x_target = ARR(0.);

  soc.setControlVariables(TUPLE(pos,col));
  
  //-- inverse kinematics to reach the targets
  arr q,dq;
  soc.getq0(q);
  for(uint t=0;t<T;t++){
    soc::bayesianIKControl(soc,dq,0);
    q += dq;
    soc.setq(q);
    //soc.reportOnState(cout); //->would generate detailed ouput on the state of all variables...
    gl.update(STRING("bayesian Inverse Kinematics: iteration "<<t));
    //gl.watch();
  }
  gl.watch("<press ENTER>");
  
  //-- planning (AICO) to generate an optimal (kinematic) trajectory
  soc.getq0(q);
  soc.setq(q);
  pos->setInterpolatedTargetsEndPrecisions(T,1e-3,1e3,0.,1e3);
  col->setInterpolatedTargetsConstPrecisions(T,1e-2,0.);
  
  q.clear();
  if(!soc.dynamic) AICO_kinematic(soc,q,40,.7,.01,0);
  else             AICO_dynamic  (soc,q,40,.7,.01,0);
  ofstream os("z.traj"); q.writeRaw(os); os.close();
  for(;;) soc.displayTrajectory(q,1,"AICO (planned trajectory)");
  
  return 0;
}
